"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const ts_morph_1 = require("ts-morph");
const start_1 = require("../../runtime/start");
const nexus_logger_1 = require("../nexus-logger");
const tsc_1 = require("../tsc");
const log = nexus_logger_1.rootLogger.child('typegen');
async function generateArtifacts(layout) {
    log.trace('start');
    const startModule = start_1.createStartModuleContent({
        registerTypeScript: {
            target: ts_morph_1.ts.ScriptTarget.ES2015,
            module: ts_morph_1.ts.ModuleKind.CommonJS,
        },
        internalStage: 'build',
        absoluteModuleImports: true,
        layout: layout,
        runtimePluginManifests: [],
    });
    const transpiledStartModule = tsc_1.transpileModule(startModule, {
        target: ts_morph_1.ts.ScriptTarget.ES2015,
        module: ts_morph_1.ts.ModuleKind.CommonJS,
    });
    const result = child_process_1.spawnSync('node', ['--eval', transpiledStartModule], {
        stdio: 'inherit',
        encoding: 'utf8',
        cwd: layout.projectRoot,
        env: Object.assign(Object.assign({}, process.env), { NEXUS_SHOULD_AWAIT_TYPEGEN: 'true', NEXUS_SHOULD_EXIT_AFTER_GENERATE_ARTIFACTS: 'true' }),
    });
    if (result.error) {
        throw new Error(`Error while trying to start the typegen process:\n\n${result.error}`);
    }
    if (result.stderr) {
        throw new Error(`Error while trying to start the typegen process:\n\n${result.stderr}`);
    }
    if (result.status !== 0) {
        throw new Error(`
      Nexus artifact generation failed with exit code "${result.status}". The following stderr was captured:

          ${result.stderr}
    `);
    }
    log.trace('done', result);
}
exports.generateArtifacts = generateArtifacts;
//# sourceMappingURL=artifact-generation.js.map