"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Lo = tslib_1.__importStar(require("lodash"));
/**
 * Create a logger.
 */
function create(rootState, path, parentContext) {
    const state = {
        // Copy as addToContext will mutate it
        pinnedAndParentContext: Lo.cloneDeep(parentContext),
        children: [],
    };
    function updateContextAndPropagate(newContext) {
        state.pinnedAndParentContext = newContext;
        state.children.forEach((child) => {
            child.onNewParentContext(state.pinnedAndParentContext);
        });
    }
    function forwardToPino(level, event, localContext) {
        // Avoid mutating the passed local context
        const context = localContext
            ? Lo.merge({}, state.pinnedAndParentContext, localContext)
            : state.pinnedAndParentContext;
        rootState.pino[level]({ path, context }, event);
    }
    const link = {
        onNewParentContext(newParentContext) {
            updateContextAndPropagate(Lo.merge(
            // Copy so that we don't mutate parent while maintaining local overrides...
            {}, newParentContext, 
            // ...this
            state.pinnedAndParentContext));
        },
    };
    const logger = {
        fatal(event, context) {
            forwardToPino('fatal', event, context);
        },
        error(event, context) {
            forwardToPino('error', event, context);
        },
        warn(event, context) {
            forwardToPino('warn', event, context);
        },
        info(event, context) {
            forwardToPino('info', event, context);
        },
        debug(event, context) {
            forwardToPino('debug', event, context);
        },
        trace(event, context) {
            forwardToPino('trace', event, context);
        },
        addToContext(context) {
            // Can safely mutate here, save some electricity...
            updateContextAndPropagate(Lo.merge(state.pinnedAndParentContext, context));
            return logger;
        },
        child: (name) => {
            const { logger: child, link } = create(rootState, path.concat([name]), state.pinnedAndParentContext);
            state.children.push(link);
            return child;
        },
    };
    return {
        logger,
        link,
    };
}
exports.create = create;
//# sourceMappingURL=logger.js.map