"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Lo = tslib_1.__importStar(require("lodash"));
const pino_1 = tslib_1.__importDefault(require("pino"));
const strip_ansi_1 = tslib_1.__importDefault(require("strip-ansi"));
const Prettifier = tslib_1.__importStar(require("./prettifier"));
/**
 * Helper to create pino instance. Aside from encapsulating some hardcoded
 * settings this is also useful because we call it from multiple places.
 * Currently when changing in/out of pretty mode and construction time.
 */
function create(opts) {
    const pino = pino_1.default({
        prettyPrint: opts.pretty.enabled,
        prettifier: (_opts) => opts.pretty.color
            ? Prettifier.create(opts.pretty)
            : // todo more performant way to do this would be give task to de-colour
                // to render function. Then it can just use some cheap if conditions.
                // Presumably strip-ansi is doing WAY more work as it has to
                // traverse EVERY string logged...
                Lo.flow(Prettifier.create(opts.pretty), strip_ansi_1.default),
        messageKey: 'event',
    }, opts.output);
    pino.level = opts.level;
    return pino;
}
exports.create = create;
//# sourceMappingURL=pino.js.map