"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const util_1 = require("util");
const utils_1 = require("../utils");
const chalk_1 = require("./chalk");
const Level = tslib_1.__importStar(require("./level"));
const Logger = tslib_1.__importStar(require("./logger"));
const Pino = tslib_1.__importStar(require("./pino"));
/**
 * Create a root logger.
 */
function create(opts) {
    var _a, _b;
    let level = opts === null || opts === void 0 ? void 0 : opts.level;
    if (level === undefined) {
        if (process.env.LOG_LEVEL) {
            level = parseFromEnvironment('LOG_LEVEL', Level.parser);
        }
        else {
            level = process.env.NODE_ENV === 'production' ? Level.LEVELS.info.label : Level.LEVELS.debug.label;
        }
    }
    const state = {};
    const loggerLink = Logger.create(state, [(_a = opts === null || opts === void 0 ? void 0 : opts.name) !== null && _a !== void 0 ? _a : 'root'], {});
    const logger = loggerLink.logger;
    logger.settings = ((newSettings) => {
        if ('pretty' in newSettings) {
            // @ts-ignore
            logger.settings.pretty = processSettingInputPretty(newSettings.pretty, logger.settings.pretty);
        }
        if ('level' in newSettings) {
            // @ts-ignore
            logger.settings.level = newSettings.level;
        }
        // sync pino
        if ('pretty' in newSettings) {
            // Pino does not support updating pretty setting, so we have to recreate it
            state.pino = Pino.create(logger.settings);
        }
        if ('level' in newSettings) {
            state.pino.level = logger.settings.level;
        }
        // sync chalk
        if ('pretty' in newSettings) {
            // todo do not assume true color support
            // https://github.com/chalk/chalk#256-and-truecolor-color-support
            chalk_1.chalk.level = newSettings.pretty ? 3 : 0;
        }
        return logger;
    });
    Object.assign(logger.settings, {
        pretty: processSettingInputPretty(opts === null || opts === void 0 ? void 0 : opts.pretty, null),
        level,
        output: (_b = opts === null || opts === void 0 ? void 0 : opts.output) !== null && _b !== void 0 ? _b : process.stdout,
    });
    state.pino = Pino.create(logger.settings);
    return logger;
}
exports.create = create;
/**
 * Run a given parser over an environment variable. If parsing fails, throw a
 * contextual error message.
 */
function parseFromEnvironment(key, parser) {
    const envVarValue = process.env[key]; // assumes env presence handled before
    const result = parser.run(envVarValue);
    if (result === null) {
        throw new Error(`Could not parse environment variable ${key} into ${parser.info.typeName}. The environment variable was: ${util_1.format(envVarValue)}. A valid environment variable must be like: ${parser.info.valid}`);
    }
    return result;
}
/**
 * Process pretty setting input.
 */
function processSettingInputPretty(pretty, previous) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    // todo no semantic to "unset back to default"
    // consider using `null` for that purpose...
    const color = (_b = (_a = (typeof pretty === 'object' ? pretty.color : undefined)) !== null && _a !== void 0 ? _a : previous === null || previous === void 0 ? void 0 : previous.color) !== null && _b !== void 0 ? _b : true;
    const enabled = (_d = (_c = (typeof pretty === 'object' ? pretty.enabled : undefined)) !== null && _c !== void 0 ? _c : previous === null || previous === void 0 ? void 0 : previous.enabled) !== null && _d !== void 0 ? _d : 
    // todo nice is-defined-but-parse-error feedback
    (((_e = process.env.LOG_PRETTY) === null || _e === void 0 ? void 0 : _e.toLowerCase()) === 'true'
        ? true
        : ((_f = process.env.LOG_PRETTY) === null || _f === void 0 ? void 0 : _f.toLowerCase()) === 'false'
            ? false
            : process.stdout.isTTY);
    const levelLabel = (_h = (_g = (typeof pretty === 'object' ? pretty.levelLabel : undefined)) !== null && _g !== void 0 ? _g : previous === null || previous === void 0 ? void 0 : previous.levelLabel) !== null && _h !== void 0 ? _h : false;
    const timeDiff = (_k = (_j = (typeof pretty === 'object' ? pretty.timeDiff : undefined)) !== null && _j !== void 0 ? _j : previous === null || previous === void 0 ? void 0 : previous.timeDiff) !== null && _k !== void 0 ? _k : true;
    if (pretty === undefined) {
        return { enabled, color, levelLabel, timeDiff };
    }
    if (pretty === true) {
        return { enabled: true, color, levelLabel, timeDiff };
    }
    if (pretty === false) {
        return { enabled: false, color, levelLabel, timeDiff };
    }
    if (typeof pretty === 'object') {
        return { enabled, color, levelLabel, timeDiff };
    }
    utils_1.casesHandled(pretty);
}
//# sourceMappingURL=root-logger.js.map