"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const common_tags_1 = require("common-tags");
const Lo = tslib_1.__importStar(require("lodash"));
const Logger = tslib_1.__importStar(require("../lib/logger"));
const Plugin = tslib_1.__importStar(require("../lib/plugin"));
const Schema = tslib_1.__importStar(require("./schema"));
const Server = tslib_1.__importStar(require("./server"));
const log = Logger.create({ name: 'app' });
/**
 * Crate an app instance
 */
function create() {
    const __state = {
        plugins: [],
        isWasServerStartCalled: false,
    };
    const server = Server.create();
    const schemaComponent = Schema.create(__state);
    const settings = {
        change(newSettings) {
            if (newSettings.logger) {
                log.settings(newSettings.logger);
            }
            if (newSettings.schema) {
                schemaComponent.private.settings.change(newSettings.schema);
            }
            if (newSettings.server) {
                server.settings.change(newSettings.server);
            }
        },
        current: {
            logger: log.settings,
            schema: schemaComponent.private.settings.data,
            server: server.settings.data,
        },
        original: Lo.cloneDeep({
            logger: log.settings,
            schema: schemaComponent.private.settings.data,
            server: server.settings.data,
        }),
    };
    const api = {
        __state,
        log: log,
        settings: settings,
        schema: schemaComponent.public,
        use(plugin) {
            if (__state.isWasServerStartCalled === true) {
                log.warn(common_tags_1.stripIndent `
          A plugin was ignored because it was loaded after the server was started
          Make sure to call \`use\` before you call \`server.start\`
        `);
            }
            __state.plugins.push(plugin);
        },
        server: {
            express: server.express,
            /**
             * Start the server. If you do not call this explicitly then nexus will
             * for you. You should not normally need to call this function yourself.
             */
            async start() {
                if (process.env.NEXUS_DISABLE_SERVER === 'true') {
                    return;
                }
                // Track the start call so that we can know in entrypoint whether to run
                // or not start for the user.
                __state.isWasServerStartCalled = true;
                const plugins = await Plugin.importAndLoadRuntimePlugins(__state.plugins);
                const schema = await schemaComponent.private.makeSchema(plugins);
                await server.setupAndStart({
                    schema,
                    plugins,
                    contextContributors: schemaComponent.private.state.contextContributors,
                });
            },
            stop() {
                return server.stop();
            },
        },
    };
    return api;
}
exports.create = create;
//# sourceMappingURL=app.js.map