"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const NexusSchema = tslib_1.__importStar(require("@nexus/schema"));
const core_1 = require("@nexus/schema/dist/core");
const common_tags_1 = require("common-tags");
const add_to_context_extractor_1 = require("../../lib/add-to-context-extractor/add-to-context-extractor");
const Layout = tslib_1.__importStar(require("../../lib/layout"));
const nexus_schema_stateful_1 = require("../../lib/nexus-schema-stateful");
const logger_1 = require("./logger");
const settings_1 = require("./settings");
function create(appState) {
    const statefulNexusSchema = nexus_schema_stateful_1.createNexusSchemaStateful();
    const state = {
        schemaPlugins: [],
        settings: {},
        contextContributors: [],
    };
    const middleware = (fn) => {
        api.public.use(NexusSchema.plugin({
            // TODO: Do we need to expose the name property?
            name: 'local-middleware',
            onCreateFieldResolver(config) {
                return fn(config);
            },
        }));
    };
    const api = {
        public: Object.assign(Object.assign({}, statefulNexusSchema.builders), { use(plugin) {
                if (appState.isWasServerStartCalled === true) {
                    logger_1.log.warn(common_tags_1.stripIndent `
            A Nexus Schema plugin was ignored because it was loaded after the server was started
            Make sure to call \`schema.use\` before you call \`server.start\`
          `);
                }
                state.schemaPlugins.push(plugin);
            },
            addToContext(contextContributor) {
                state.contextContributors.push(contextContributor);
            },
            middleware }),
        private: {
            state: state,
            makeSchema: async (plugins) => {
                const nexusSchemaConfig = settings_1.mapSettingsToNexusSchemaConfig(plugins, state.settings);
                nexusSchemaConfig.types.push(...statefulNexusSchema.state.types);
                nexusSchemaConfig.plugins.push(...state.schemaPlugins);
                const { schema, missingTypes, finalConfig } = core_1.makeSchemaInternal(nexusSchemaConfig);
                if (nexusSchemaConfig.shouldGenerateArtifacts === true) {
                    const devModeLayout = await Layout.loadDataFromParentProcess();
                    if (!devModeLayout) {
                        throw new Error('Layout should be defined when should gen artifacts is true. This should not happen.');
                    }
                    const contextExtractionPromise = add_to_context_extractor_1.runAddToContextExtractorAsWorkerIfPossible(devModeLayout.data);
                    const typegenPromise = nexus_schema_stateful_1.writeTypegen(schema, finalConfig, state.settings.rootTypingsGlobPattern, devModeLayout);
                    // Await promise only if needed. Otherwise let it run in the background
                    if (process.env.NEXUS_SHOULD_AWAIT_TYPEGEN === 'true') {
                        await Promise.all([typegenPromise, contextExtractionPromise]);
                    }
                    if (nexusSchemaConfig.shouldExitAfterGenerateArtifacts) {
                        process.exit(0);
                    }
                }
                /**
                 * Assert that there are no missing types after running typegen only
                 * so that we don't block writing the typegen when eg: renaming types
                 */
                NexusSchema.core.assertNoMissingTypes(schema, missingTypes);
                if (statefulNexusSchema.state.types.length === 0) {
                    logger_1.log.warn(Layout.schema.emptyExceptionMessage());
                }
                return schema;
            },
            settings: {
                data: state.settings,
                change(newSettings) {
                    settings_1.changeSettings(state.settings, newSettings);
                },
            },
        },
    };
    return api;
}
exports.create = create;
//# sourceMappingURL=schema.js.map